#!/usr/local/bin/perl -w
#
#  Colyleft CryptoManceR 
#  ( http://users.livejournal.com/cryptomancer_/ , 
#    http://cryptomancer.narod.ru/) 
#
#  Either Perl license or GNU GPL at your wish
#
#  Usage: gpx2exif track.gpx *.jpg
#  (Use http://www.gpsbabel.org/ to convert your tracks to GPX format)
#
#  Partially based on Geo::Track::Log
#

use Data::Dumper; # for debug only

use File::Glob ':globally';
use Date::Parse;
require Image::ExifTool; my $exifTool = new Image::ExifTool;
require XML::Simple;

my $xs = new XML::Simple(
		ForceArray => [ 'trk', 'trkseg', 'trkpt' ],
		KeyAttr => [],NormaliseSpace => 2);
my @points; my $NumPoints = -1; GPXtoPoints($ARGV[0]); #print Dumper(@points);
my ($MinTime) = split(',',$points[0]);
my ($MaxTime) = split(',',$points[$NumPoints]);

	#print "\n Min=",$MinTime," Max=",$MaxTime," Num=",$NumPoints,"\n"; # Debug

my @pictures = <$ARGV[1]>;

foreach $picture ( @pictures ) {
 if ( 0 == $exifTool->ExtractInfo($picture)) {next;};
 $PicTime = str2time($exifTool->GetValue('DateTimeOriginal'));

 if ($PicTime < $MinTime) {print "$picture older than $ARGV[0], skip.\n"; next;};
 if ($PicTime > $MaxTime) {print "$picture newer than $ARGV[0], skip.\n"; next;};

 ($p1, $p2) = BetweenPoints(0,$NumPoints);  # print $p1, " " ,$p2, "\n"; # Debug only

 ($time1,$lat1,$lon1,$alt1) = split(',',$points[$p1]);
 ($time2,$lat2,$lon2,$alt2) = split(',',$points[$p2]);

 $Weight = ($time2 - $PicTime) / ($time2 - $time1);
 $lat = $lat1*$Weight + $lat2*(1-$Weight);
 $lon = $lon1*$Weight + $lon2*(1-$Weight);
 $alt = $alt1*$Weight + $alt2*(1-$Weight);

	 #print $lat1,',',$lat2,',',$lat; # Debug

 $latRef = 'N';
 if ($lat < 0) { $latRef = 'S'; $lat = 0-$lat; }

 $lonRef = 'E';
 if ($lon < 0) { $lonRef = 'W'; $lon = 0-$lon; }

 $altRef = '0';
 if ($alt < 0) { $altRef = '1'; $alt = 0-$alt; }

 $exifTool->SetNewValue('GPSLatitude',$lat);
 $exifTool->SetNewValue('GPSLongitude',$lon);
 $exifTool->SetNewValue('GPSLatitudeRef',$latRef);
 $exifTool->SetNewValue('GPSLongitudeRef',$lonRef);
# $exifTool->SetNewValue('GPSAltitudeRef',$altRef); # Broken ?
 $exifTool->SetNewValue('GPSAltitude',$alt);

 $exifTool->WriteInfo($picture);

}

sub BetweenPoints {
 my $p1 = $_[0]; my $p2 = $_[1];

 # narrowed to 2 points ?
 if ( 1 == abs($p2 - $p1)) { return ($p1, $p2);};

 my $pMid = int( ($p1+$p2) /2 );

 $tMid = GetPointTime($pMid); # loop cleaning
 
 if ($PicTime < $tMid ) 
  { return BetweenPoints($p1,$pMid); }
 else
  { return BetweenPoints($pMid,$p2); }
}

sub GetPointTime { my @t = split(',',$points[$_[0]]); return $t[0];}

sub GPXtoPoints {
my $gpx = $xs->XMLin($_[0]);
for my $trk (@{$gpx->{trk}}) {
  for my $seg (@{$trk->{trkseg}}) {
    for my $pt (@{$seg->{trkpt}}) {
      $pt->{time} =~ y/TZ/ /d; # "2004-08-29T01:44:11Z" -> "2004-08-29 01:44:11"

      my $s = str2time($pt->{time}).','.$pt->{lat}.','.$pt->{lon}.','.$pt->{ele};
      push(@points,$s);
      $NumPoints++;
    }
  }
}}